import java.awt.Component;
import java.net.MalformedURLException;
import java.net.URL;

import javax.help.HelpBroker;
import javax.help.HelpSet;
import javax.swing.JComponent;

/*
 * Cette Classe gre l'aide dans le programme,
 * en appellant JavaHelp 2.0
 */

/**
 * @author Sebastian Gfeller, Andreas Jaggi
 * @created Jun 7, 2004
 * @version AideManager.java,v 1.10 2004/06/26 10:10:50 sgfeller Exp
 * @copyright Sebastian Gfeller, Andreas Jaggi 2004
 * @license http://opensource.org/licenses/gpl-license.php GNU Public License
 */
public class AideManager implements Constantes {
	
	private HelpBroker hb = null;
	private HelpSet hs = null;
	final String HELPSETFILE = "bombhelp.hs";
	final String HELPSETPATH = "help/"; 
	
	public AideManager() {
		super();
	}
	
	/** cre un URL d'ou se trouve l'aide en utilisant les constantes HELPSETPATH et HELPSETFILE (hardcoded)
	 * @param helpset Le fichier helpset qu'on veut afficher.
	 * @return true si on a cr le HelpBroker necessaire pour afficher l'aide
	 */
	private boolean prepareHelp(String helpset) {
		boolean success = true; // let's face it: multiple return statements are evil
		
		URL hsURL = null;
		
		try {
			// URL avec prefix "jar:" pour utilisation dans JavaHelp...
			// ne jamais utiliser toString() si on veut reellement getFile() !
			hsURL = new URL("jar:"+getClass().getResource(HELPSETPATH+helpset).getFile());
		} catch (MalformedURLException e) {
			System.out.println(hsURL.toString()+" est mal form");
		}
		

		try {
			/* trouve le fichier .hs dans le JAR
			 * Si seulement j'aurais un peu plus de temps, ce JavaHelp me semble interessant
			 */
			hs = new HelpSet(null, hsURL);
		} catch ( Exception ee) {
			if ( DEBUG ) {
				ee.printStackTrace();
			}
			success = false;
		} 
		
		if(success) {
			try {
				hb = hs.createHelpBroker();
			} catch (Exception ee) {
				if ( DEBUG ) {
					ee.printStackTrace();
				}
				success = false;
			}
		}
		
		return success;
	}
	
	/** appelle l'aide avec la page d'entre. Signalise au HelpBroker que l'aide a t initialise
	 */
	
	public void afficherAide() {
		
		if(prepareHelp(HELPSETFILE)) {
			if( DEBUG ) {
				System.out.println("Afficher Aide...");
			}
			hb.setDisplayed(true);
		} else {
			if ( DEBUG ) {
				System.out.println("Problme avec l'affichage de l'aide.");
			}
		}  
		
	}

	/** Ajoute l'aide contextuelle par ce contexte. On a seulement un helpset
	 * et plusieurs componentes qui l'appellent, alors a fait sens de abstraire
	 * a.
	 * @param eContexte l'element qui a une aide contextuelle
	 * @param cible le cible dans le HelpSet
	 */
	public void ajouterContexte(Component eContexte, String cible) {
		hb.enableHelp(eContexte,cible,hs);
	}
}
