
/**
 * Author:      Frdric Viela
 * Date:        2004
 *
 * AStar:	Andreas Jaggi
 */


import java.awt.Graphics;
import java.awt.Image;
import java.awt.Point;

/**
* Reprsente une case du niveau, avec:
* son type
* son bonus
* son image
* sa position
*/

public class Block implements Sol, Constantes {
    // La zone de jeu o afficher les Blocks
    public static ZoneJeu zoneJeu;
    // Le type du Block
    // - Vaut le type du Block si aucun bonus
    // - Vaut 1 si il y a un bonus
    int type;
    // Le bonus du block
    // - Vaut 0 si aucun bonus
    int bonus;
    // L'image du Block, dpend du type
    Image image;
    //  La position du Block dans le Niveau
    Point position;
    

		/* variables utilis pour AStar (ajaggi) */

		/**
		* Couts du block
		*/
		private double couts = COUTS_NORMAL;

		/**
		* Distance du start du block
		*/
		private double distDuStart = -100;

		/**
		* Indique si'il y a une bombe
		*/
		public boolean bombe = false;

		/**
		* Indique si le block est en porte d'une bombe
		*/
		public int bombevoisin = 0;



		/* fonctions utilis pour AStar (ajaggi) */

		/**
		* Donne la position du block
		* @return La position du block
		*/
		public Point getPosition ( ) {
			return new Point(this.position.y / OFFSET, this.position.x / OFFSET);
		}

		/**
		 * Calcule une sorte de distance entre le block et le start
		 * @param distJusqueIci Distance du start jusqu'avant ce block
		 */
		public void ajouteADistDuStart ( double distJusqueIci ) {

			/* on recoit le premier fois une indication de la distance,
			 * alors on l'accepte */
			if ( this.distDuStart == -100 ) {
				this.distDuStart = distJusqueIci + this.couts;
				return;
			}

			/* on a dj une distance mais on prend la plus courte */
			if ( distJusqueIci + this.couts < this.distDuStart ) {
				this.distDuStart = distJusqueIci + this.couts;
			}
		}

		/**
		 * Donne les couts du block
		 * @return Les couts du block
		 */
		public double getCouts () {
			return this.couts;
		}

		/**
		 * Sets couts du block
		 * @param c Couts
		 */
		public void setCouts ( double c ) {
			this.couts = c;
		}

		/**
		 * Indique si le block est intraversable ou pas
		 * @return Intraversable
		 */
		public boolean isIntraversable ( ) {
			return (this.couts == COUTS_INTRAVERSABLE || this.couts == COUTS_BOMBE);
		}

		/**
		 * Sets intraversable
		 * @param intraversable Intraversable
		 */
		public void setIntraversable ( boolean intraversable ) {
			if ( intraversable ) {
				this.couts = COUTS_INTRAVERSABLE;
			} else {
				this.couts = COUTS_NORMAL;
			}
		}

		/**
		 * Resets le block. (e.g. les couts et la distance du start)
		 */
		public void resetBlock ( ) {
			this.couts = COUTS_NORMAL;

			if ( this.bombe ) {
				this.couts = COUTS_BOMBE;
			}

			if ( this.bombevoisin > 0) {
				this.couts = COUTS_BOMBEVOISIN;
			}

			this.distDuStart = -100;
		}

		/**
		 * Donne la distance entre le block start et ce block
		 * @param start Block start
		 */
		public double getDistDuStart ( Block start ) {
			if ( start == this ) {
				return 0;
			} 

			if ( this.isIntraversable() ) {
				return COUTS_INTRAVERSABLE;
			}

			return this.distDuStart;
		}

		/**
		 * Calcule les couts en dpendance du type du block
		 */
		public void setTypeCouts ( ) {

			this.couts = COUTS_NORMAL;

			if ( this.bombe ) {
				this.couts = COUTS_BOMBE;
			}

			if ( this.bombevoisin > 0) {
				this.couts = COUTS_BOMBEVOISIN;
			}

			switch ( this.type ) {
				case MUR:
				case MOU:
				case INDESTRUCTIBLE:
					this.couts = COUTS_INTRAVERSABLE;
					break;
				case BONUSFLAMME:
				case BONUSBOMBE:
				case BONUSVITESSE:
					this.couts = COUTS_BONUS;
					break;
				case MALUSLENTEUR:
					this.couts = COUTS_MALUS;
					break;
				case SOL:
				default:
			}
		}



    /**
    * Constructeur
    * @param image L'image du Block
    * @param type Le type du block
    * @param i La ligne du Block dans le niveau
    * @param j La colonne du Block dans le niveau
    */
       
    public Block(Image image, int type, int i, int j) {
        this.image = image;
        position = new Point(i * OFFSET, j * OFFSET);
        
        // Suivant le type, on gre un bonus ou non
        if (type > 3) {
            this.bonus = type;
            this.type = 1;
        }
        else {
            this.bonus = 0;
            this.type = type;
        }

	/* calculer les couts du block */
	this.setTypeCouts();
    }
    
    /**
    * Initialise la zone de jeu
    */
       
    public static void init(ZoneJeu zj) {
        zoneJeu = zj;
    }
    
    /**
    * Fait exploser un Block
    * @param image Tableau des images
    * Retourne true si un Block a t dtruit, false sinon
    */
       
    public boolean explose(Image image[]) {
        // Suivant le type du Block, on change la valeur de type
        switch(type) {
            case SOL:
                return false;
            case MOU:
                type--;
                setImage(image[type]);
                break;
            case MUR:
                type--;
                setImage(image[type]);
                break; 
            case INDESTRUCTIBLE:
                return true;
            case BONUSFLAMME:
                type=0;
                bonus=0;
                break;
            case BONUSBOMBE:
                type=0;
                bonus=0;
                break;
            case MALUSLENTEUR:
                type=0;
                bonus=0;
                break;
            case BONUSVITESSE:
                type=0;
                bonus=0;
                break;
        }
        
        // Si destruction d'un bonus, on remplace par le sol
        if (bonus > 3) {
            type = bonus;
            setImage(image[bonus]);
        }
        else
            setImage(image[type]);

        paint(zoneJeu.gfond);

	this.resetBlock();
	this.setTypeCouts();

        return true;
    }
    
    /**
    * Change l'image du Block
    * @param image La nouvelle image du Block
    */
       
    void setImage(Image image) {
        this.image = image;
    }
    
    /**
    * Repaint le Block dans le fond de la zone de jeu
    * @param g Contexte graphique du fond de la zone de jeu
    */
       
    public void paint(Graphics g) {
        g.drawImage(image, position.x, position.y, zoneJeu);
    }
}
