import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.KeyStroke;

import bombmapper.BombMapper;


/**
 * Un interface plus joli pour lancer les diffrents types de jeu et l'diteur
 *
 * @author	Andreas Jaggi, Sebastian Gfeller
 * @created	June 18, 2004
 * @version	StartWindow.java,v 1.13 2004/06/27 14:45:03 sgfeller Exp
 * @copyright	Andreas Jaggi, Sebastian Gfeller 2004
 * @license     http://opensource.org/licenses/gpl-license.php GNU Public License
 */

class StartWindow extends JFrame implements ActionListener {

	/* d'abord on definit les components pour le menu simple */
	/**
	 * Bouton Lancer
	 */
	private JButton lanceJeu = new JButton("Lancer");

	/**
	 * Ensemble des boutons radio
	 */
	private ButtonGroup typeJeu = new ButtonGroup();

	/**
	 * Bouton radio mode homme vs ordinateur
	 */
	private JRadioButton modeSingle = new JRadioButton("Homme vs. ordinateur", true);

	/**
	 * Bouton radio mode multijoueur avec serveur
	 */
	private JRadioButton modeServeur = new JRadioButton("Multijoueur (avec serveur)");

	/**
	 * Bouton radio mode multijoueur client
	 */
	private JRadioButton modeClient = new JRadioButton("Multijoueur (client)");

	/**
	 * Bouton radio mode simulation
	 */
	private JRadioButton modeSimul = new JRadioButton("Simulation");

	/**
	 * Label nom
	 */
	private JLabel labelNom = new JLabel("Nom");

	/**
	 * JTextField pour le nom du joueur
	 */
	private JTextField fieldNom = new JTextField("Nom");

	/**
	 * JPanel pour le menu du centre
	 */
	private JPanel centerPanel = new JPanel();

	/**
	 * JPanel avec l'image arrire
	 */
	private JPanel bgPanel = new JPanel();

	/* et ensuite les components pour le menu avanc */
	/**
	 * Menubar du StartWindow
	 */
	private JMenuBar jmb = new JMenuBar();
	
	/**
	 * Menu jeu
	 */
	private JMenu jeux = new JMenu("Jeu");

	/**
	 * Menuitem pour quitter 
	 */
	private JMenuItem itemQuitter = new JMenuItem("Quitter");

	/**
	 * Menuitem pour lancer homme vs ordinateur
	 */
	private JMenuItem lancerSingle = new JMenuItem("Homme vs. ordinateur");

	/**
	 * Menuitem pour lancer multijoueur avec serveur
	 */
	private JMenuItem lancerServeur = new JMenuItem("Multijoueur (avec serveur)");

	/**
	 * Menuitem pour lancer multijoueur client
	 */
	private JMenuItem lancerClient = new JMenuItem("Multijoueur (client)");

	/**
	 * Menuitem pour lancer la simulation
	 */
	private JMenuItem lancerSimul = new JMenuItem("Simulation");

	/**
	 * Menu diteur
	 */
	private JMenu bombmapperMenu = new JMenu("BombMapper");

	/**
	 * Menuitem pour lancer le BombMapper
	 */
	private JMenuItem lancerBombmapper = new JMenuItem("Lancer le BombMapper");

	/**
	 * Menu aide
	 */
	private JMenu aide = new JMenu("Aide");

	/**
	 * Menuitem pour lancer l'aide
	 */
	private JMenuItem indexAide = new JMenuItem("Aide bomberman"); // la vraie "aide"

	/**
	 * Menuitem apropos
	 */
	private JMenuItem apropos = new JMenuItem("A propos");

	/**
	 * Menuitem aufait
	 */
	private JMenuItem aufait = new JMenuItem("Au fait");

	/**
	 * AideManager pour afficher l'aide
	 */
	private AideManager aideManager;

	/**
	 * l'image arrire
	 */
	private ImageIcon bgImage;

	/**
	 * nom du fichier jar
	 */
	private String jarname;

	/**
	 * nom du joueur
	 */
	private String nom;


	/**
	 * Constructeur de StartWindow
	 */
	public StartWindow ( ) {
		super("Bomberman 2004");

		/* dfini le nom du jar */
		this.jarname ="bomberman.jar";

		/* cre une instance de AideManager pour cette fentre */
		this.aideManager = new AideManager();

		/* charge l'image arrire */
		this.bgImage = new ImageIcon(getClass().getResource("gfx/bg.png"));

		/* ajoute les components au menubar */
		this.populerMenuBar(this.jmb);

		/* ajoute le menubar au fentre */
		this.setJMenuBar(this.jmb);

		/* cre le panel avec l'image arrire */
		this.bgPanel = new JPanel ( ) {
			public void paintComponent ( Graphics g ) {
				g.drawImage(bgImage.getImage(), 0, 0, this);
				super.paintComponent(g);
			}
		};

		/* ajoute les components au menu du centre */
		this.populerCenterMenu(this.centerPanel);

		/* adapte le bgPanel  nos besoins */
		this.bgPanel.setOpaque(false);
		this.bgPanel.setSize(600, 600);
		this.bgPanel.setLayout(new FlowLayout());

		/* ajoute le menu du centre au bgPanel */
		this.bgPanel.add(this.centerPanel, BorderLayout.CENTER);

		/* ajoute le bgPanel au fentre */
		this.getContentPane().add(this.bgPanel);

		this.setSize(600,600);

		/* centre le fentre dans l'cran */
		this.center();

		/* nom par dfaut du joueur */
		this.nom = "Nom";
	}

	/**
	 * Ajoute des components du menu du centre a un JPanel donn
	 *
	 * @param centerMenu JPanel pour ajouter les components
	 */
	private void populerCenterMenu ( Container centerMenu ) {

		/* ajoute les boutons radio  l'ensemble typeJeu */
		this.typeJeu.add(this.modeSingle);
		this.typeJeu.add(this.modeServeur);
		this.typeJeu.add(this.modeClient);
		this.typeJeu.add(this.modeSimul);

		/* ajoute un ActionListener au bouton lancer */
		this.lanceJeu.addActionListener(this);

		/* adapte le layout du centermenu  nos besoins */
		centerMenu.setLayout(new BoxLayout(centerMenu, BoxLayout.PAGE_AXIS));

		/* fait tous les lments transparent, sauf le bouton et le JTextField */
		this.modeSingle.setOpaque(false);
		this.modeServeur.setOpaque(false);
		this.modeClient.setOpaque(false);
		this.modeSimul.setOpaque(false);
		this.centerPanel.setOpaque(false);
		this.labelNom.setOpaque(false);

		/* ajoute les components au centerMenu, 
		 * avec des JPanel transparents pour crer un peu d'space entre eux */
		this.addCentered(this.blankPanel(30,50), centerMenu);
		this.addCentered(this.labelNom, centerMenu);
		this.addCentered(this.blankPanel(30,5), centerMenu);
		this.addCentered(this.fieldNom, centerMenu);
		this.addCentered(this.blankPanel(30,10), centerMenu);
		this.addCentered(this.modeSingle, centerMenu);
		this.addCentered(this.modeServeur, centerMenu);
		this.addCentered(this.modeClient, centerMenu);
		this.addCentered(this.modeSimul, centerMenu);
		this.addCentered(this.blankPanel(30,10), centerMenu);
		this.addCentered(this.lanceJeu, centerMenu);
	}

	/**
	 * Ajoute un JComponent  un Container de faon centr
	 *
	 * @param comp JComponent  ajouter
	 * @param container Container
	 */
	private void addCentered ( JComponent comp, Container container ) {
		comp.setAlignmentX(Component.CENTER_ALIGNMENT);
		container.add(comp);
	}

	/**
	 * Ajoute les components au JMenuBar
	 *
	 * @param menuBar Le JMenuBar
	 */
	private void populerMenuBar ( Container menuBar ) {

		/* Pour les raccourcis clavier */
		int mod = Toolkit.getDefaultToolkit().getMenuShortcutKeyMask();


		/* ajoute "homme vs ordinateur" avec tooltip et raccourci clavier au menu jeu */
		this.lancerSingle.setToolTipText("Lance une partie homme vs. ordinateur");
		this.lancerSingle.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_1,mod));
		this.lancerSingle.addActionListener(this);
		this.jeux.add(this.lancerSingle);

		/* ajoute "multijoueur avec serveur" avec tooltip et raccourci clavier au menu jeu */
		this.lancerServeur.setToolTipText("Lance une partie multijoueur (avec serveur)");
		this.lancerServeur.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_2,mod));
		this.lancerServeur.addActionListener(this);
		this.jeux.add(this.lancerServeur);

		/* ajoute "multijoueur client" avec tooltip et raccourci clavier au menu jeu */
		this.lancerClient.setToolTipText("Lance une partie multijoueur (client)");
		this.lancerClient.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_3,mod));
		this.lancerClient.addActionListener(this);
		this.jeux.add(this.lancerClient);

		/* ajoute "simulation" avec tooltip et raccourci clavier au menu jeu */
		this.lancerSimul.setToolTipText("Lance la simulation du jeu");
		this.lancerSimul.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_4,mod));
		this.lancerSimul.addActionListener(this);
		this.jeux.add(this.lancerSimul);

		/* ajoute "quitter" avec tooltip et raccourci clavier au menu jeu */
		this.itemQuitter.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q,mod));
		this.itemQuitter.setToolTipText("Quitte le programme");
		this.itemQuitter.addActionListener(this);
		this.jeux.add(this.itemQuitter);

		/* ajoute le menu jeu au menubar */
		menuBar.add(this.jeux);

		/* ajoute "lancer bombmapper" avec tooltip et raccourci clavier au menu bombmapper */
		this.lancerBombmapper.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_B,mod));
		this.lancerBombmapper.setToolTipText("Lance le BombMapper");
		this.lancerBombmapper.addActionListener(this);
		this.bombmapperMenu.add(this.lancerBombmapper);

		/* ajoute le menu bombmapper au menubar */
		menuBar.add(this.bombmapperMenu);

		/* ajoute un Mnemonic au menu aide */
		this.aide.setMnemonic(KeyEvent.VK_A);

		/* ajoute "afficher aide" avec tooltip et raccourci clavier au menu aide */
		this.indexAide.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_I,mod));
		this.indexAide.setToolTipText("Affiche le guide de l'utilisateur");
		this.indexAide.addActionListener(this);
		this.aide.add(this.indexAide);

		/* ajoute "apropos" avec tooltip et raccourci clavier au menu aide */
		this.apropos.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P,mod));
		this.apropos.setToolTipText("Apropos Bomberman: Sur les auteurs");
		this.apropos.addActionListener(this);
		this.aide.add(this.apropos);

		/* ajoute "aufait" avec tooltip et raccourci clavier au menu aide */
		this.aufait.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_U,mod));
		this.aufait.setToolTipText("Donne-moi une note");
		this.aufait.addActionListener(this);
		this.aide.add(this.aufait);

		/* ajoute le menu aide au menubar */
		menuBar.add(this.aide);
	}

	/**
	 * Cre un JPanel transparent avec une dimension spcific
	 *
	 * @param width largeur
	 * @param height hauteur
	 *
	 * @return JPanel transparent
	 */
	private JPanel blankPanel ( final int width, final int height ) {
		return new JPanel ( ) {{
				setSize(width, height);
				setPreferredSize(new Dimension(width, height));
				setOpaque(false);
			}};
	}

	/**
	 * Centre le StartWindow dans l'cran
	 */
	private void center () {
		this.setLocation((Toolkit.getDefaultToolkit().getScreenSize().width - this.getSize().width)/2, (Toolkit.getDefaultToolkit().getScreenSize().height - this.getSize().height)/2);
	}

	/**
	 * Attend une certaine dur
	 *
	 * @param dure Secondes  attendre
	 */
	private static void attend ( int dure ) {
		try {
			Thread.sleep(1000*dure);
		} catch ( Exception e ) {
		}
	}

	/**
	 * Lance la simulation
	 */
	private void lanceSimul ( ) {
		/* cre le serveur, mais sans l'afficher */
		BServeur serveurLocal = new BServeur("10000","1", false);
		attend(4);

		Runtime executer = Runtime.getRuntime();

		Process[] processes = new Process[2];

		try {
			/* xecute le jar avec l'option de dmarrer un client ordinateur */
			processes[0] = executer.exec("java -jar " + this.jarname + " ordinateur lancer");
			attend(4);

			/* xecute le jar avec l'option de dmarrer un client ordinateur sans l'afficher */
			processes[1] = executer.exec("java -jar " + this.jarname + " ordinateur paslancer");
		} catch ( Exception ex ) {
			System.exit(0);
		}
	}

	/**
	 * Lance un client avec le nom du joueur spcifi
	 *
	 * @param nom Nom du joueur
	 */
	private void lanceClient ( String nom ) {
		BClient clientHumain = new BClient("humain", "localhost", "10000", nom, "false");
	}

	/**
	 * Lance un serveur et un client avec le nom du joueur spcifi
	 *
	 * @param nom Nom du joueur
	 */
	private void lanceServeur ( String nom ) {
		/* lance le serveur sans l'afficher */
		BServeur serveurLocal = new BServeur("10000","1", false);
		attend(4);

		/* lance le client */
		BClient clientHumain = new BClient("humain", "localhost", "10000", nom, "false");
	}

	/**
	 * Lance un jeu homme vs ordinateur
	 *
	 * @param nom Nom du joueur
	 */
	private void lanceSinglePlayer ( String nom ) {
		/* lance le serveur sans l'afficher */
		BServeur serveurLocal = new BServeur("10000","1", false);
		attend(4);

		Runtime executer = Runtime.getRuntime();

		Process[] processes = new Process[2];

		try {
			/* xecute le jar avec l'option de dmarrer un client ordinateur sans l'afficher */
			processes[0] = executer.exec("java -jar " + this.jarname + " ordinateur lancer");
			attend(4);

			/* xecute le jar avec l'option de dmarrer un client humain */
			processes[1] = executer.exec("java -jar " + this.jarname + " humain paslancer " + nom);
		} catch ( Exception ex ) {
			System.exit(0);
		}
	}

	/**
	 * L'action perform si on clique sur un bouton ou choisit quelque chose dans le menu.
	 * on cherche la source qui a appel actionPerformed et fait quelque chose dependant de
	 * a.
	 *
	 * @param e ActionEvent
	 */
	public void actionPerformed ( ActionEvent e ) {

		/* si le nom entr est plus long que 1 caractre,
		 * on le charge dans this.nom */
		if ( this.fieldNom.getText().length() > 1 ) {
			this.nom = this.fieldNom.getText();
		}
			
		/* on est appel par le bouton lancer du menu du centre */
		if ( e.getSource() == this.lanceJeu ) {
			
			/* cache le StartWindow */
			this.setVisible(false);
			
			/* on doit dterminer lequle des boutons radio est choisi */
			if ( this.modeSimul.isSelected() ) {
				/* lance la simulation */
				this.lanceSimul();
			} else if ( this.modeClient.isSelected() ) {
				/* lance multijoueur client */
				this.lanceClient(nom);
			} else if ( this.modeServeur.isSelected() ) {
				/* lance multijoueur avec serveur */
				this.lanceServeur(nom);
			} else {
				/* lance homme vs ordinateur */
				this.lanceSinglePlayer(nom);
			}

		} else if ( e.getSource() == indexAide ) {
			/* affiche l'aide */
			this.aideManager.afficherAide();

		} else if ( e.getSource() == this.apropos ) {
			/* affiche apropos */
			About aproposfenetre = new About(About.WINDOW_APROPOS);
			aproposfenetre.setVisible(true);

		} else if ( e.getSource() == this.aufait ) {
			/* affiche aufati */
			About aufaitfenetre = new About(About.WINDOW_AUFAIT);
			aufaitfenetre.setVisible(true);

		} else if ( e.getSource() == this.lancerSingle ) {
			/* lance homme vs ordinateur */
			this.lanceSinglePlayer(this.nom);
			this.setVisible(false);

		} else if ( e.getSource() == this.lancerServeur ) {
			/* lance multijoueur avec serveur */
			this.lanceServeur(this.nom);
			this.setVisible(false);

		} else if ( e.getSource() == this.lancerClient ) {
			/* lance multijoueur client */
			this.lanceClient(this.nom);
			this.setVisible(false);

		} else if ( e.getSource() == this.lancerSimul ) {
			/* lance la simulation */
			this.lanceSimul();
			this.setVisible(false);

		} else if ( e.getSource() == this.lancerBombmapper ) {
			/* lance l'diteur */
			BombMapper bm = new BombMapper();
			this.setVisible(false);

		} else if ( e.getSource() == this.itemQuitter ) {
			/* quitte le programme */
			System.exit(0);
		}
	}

	/**
	 * Lance une instance humain du client avec des paramtres spcifique
	 *
	 * @param lancer Indique si on doit dmarrer le jeu
	 * @param nom Le nom du joueur
	 */
	private static void lanceHumain ( boolean lancer, String nom) {
		BClient clientHumain = new BClient("humain", "localhost", "10000", nom, false, true);

		/* si c'est indiqu, on dmarre le jeu */
		if ( lancer ) {
			attend(8);

			try {
				clientHumain.lancerJeu();
				clientHumain.out.writeUTF("go");
			} catch ( Exception ex ) {
				System.exit(0);
			}
		}
	}

	/**
	 * Donne un nom particulier pour le jouoeur ordinateur
	 * 
	 * @return Nom
	 */
	private static String creeNom ( ) {
		String[] noms = new String[3];
		noms[0] = "Bombmaster";
		noms[1] = "Bomberman";
		noms[2] = "Ordinateur";

		/* on choisit un nom en dpendance des milliscondes */
		return noms[(int)(System.currentTimeMillis() % 3)];
	}

	/**
	 * Lance une instance ordinateur du client avec des paramtres spcifiques
	 *
	 * @param lancer Indique si on doit dmarrer la partie
	 */
	private static void lanceOrdinateur ( boolean lancer ) {
		BClient clientOrdinateur = new BClient("ordinateur", "localhost", "10000", creeNom(), false, !lancer);

		/* si c'est indiqu, on dmarre le jeu */
		if ( lancer ) {
			attend(8);

			try {
				clientOrdinateur.lancerJeu();
				clientOrdinateur.out.writeUTF("go");
			} catch ( Exception ex ) {
				System.exit(0);
			}
		}
	}

	/**
	 * Mthode main qui est appel du jar avant toutes les autres mthodes
	 *
	 * @param args Options de la ligne de commande
	 */
	public static void main ( String[] args ) {
		String action = "";

		if ( args.length != 0 ) {
			action = args[0];
		}

		/* s'il y a plus que 2 arguments, et le premier argument est gal  humain, 
		 * alors on doit lancer un client humain, le nom du joueur se trouve dans le troisime argument */
		if ( args.length > 2 && action.equals("humain") ) {
			/* si le deuxime argument est gal  lancer, alors il
			 * faut aussi dmarrer le jeu */
			if ( args[1].equals("lancer") ) {
				lanceHumain(true, args[2]);
			} else {
				lanceHumain(false, args[2]);
			}
		/* s'il y a plus que 1 arguments et le premier argument est gal  ordinateur,
		 * alors il faut lancer un client ordinateur */
		} else if ( args.length > 1 && action.equals("ordinateur") ) {
			/* si le deuxime argument est gal  lancer, alors il
			 * faut aussi dmarrer le jeu */
			if ( args[1].equals("lancer") ) {
				lanceOrdinateur(true);
			} else {
				lanceOrdinateur(false);
			}
		} else {
			/* Dans tous les autres cas on affiche le StartWindow */
			StartWindow startWindow = new StartWindow();
			startWindow.setVisible(true);
		}
	}
}
