package bombmapper;

import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.ObjectInputStream;

/**
 * @author Sebastian Gfeller, Andreas Jaggi
 * @created Jun 15, 2004
 * @version LevelReader.java,v 1.8 2004/06/27 16:28:26 sgfeller Exp
 * @copyright Sebastian Gfeller, Andreas Jaggi 2004
 * @license http://opensource.org/licenses/gpl-license.php GNU Public License
 */
public class LevelReader extends Level {	
	// Fichier
	private FileInputStream levelInputStream;
	// entre
	private BufferedInputStream bufferedLevelInputStream;
	private ObjectInputStream objectLevelInputStream;
	
	/** On essaie d'ouvrir un fichier qui a le nom entr dans le repertoire
	 * "level". Si a ne marche pas, on cre un nouveau fichier avec une
	 * carte sans objets
	 * @throws ClassNotFoundException
	 */
	public LevelReader(String levelname) throws ClassNotFoundException,IOException,FileNotFoundException {
		super(levelname);
		
		/* Si on ne trouve pas le fichier, on le remplit avec les dfauts */
		try {
			levelInputStream = new FileInputStream(this.levelname);
			bufferedLevelInputStream = new BufferedInputStream(levelInputStream);
			objectLevelInputStream = new ObjectInputStream(bufferedLevelInputStream);
			
			nomFichier = (String[]) objectLevelInputStream.readObject();
			type = (int[][]) objectLevelInputStream.readObject();
			
			objectLevelInputStream.close();
			bufferedLevelInputStream.close();
			
			
			// On a lu les donnes. Alors, remplissons la carte
			remplirCarte(type);
		} catch (FileNotFoundException e) {
			// Alors, on n'a pas trouv le fichier. Mais on ne veut aussi pas crer un. Alors on remplit
			// la carte avec les valeurs par defaut
			remplirCarteValeursDefauts();
		}
	}
	
	/** Appele sans arguments, le LevelReader cre une carte vide dans la mmoire, a enregistrer plus tard.
	 */
	public LevelReader() {
		super(null); // pas de nom de fichier jusqu' maintenant
		remplirCarteValeursDefauts();
	}

	/** Remplir la carte:
	 * On cre un tableau avec des Blocs. Pour toute position
	 * de type on remplit le bloc dans le tableau avec son type,
	 * le nom du fichier Terrain et le nom du fichier bonus.
	 * @param type La table avec les types des blocs.
	 */
	private void remplirCarte(int[][] type) {
		carte = new Block[type.length][type[0].length];
		
		// noms de fichiers plus necessaire, on veut changer a dynamiquement
		for(int y=0;y<carte.length;y++) {
			for(int x=0;x<carte[0].length;x++) {
				carte[y][x] = new Block(type[y][x]);
			}
		}
	}

	/**
	 * @return Le nom de la carte qu'on gre avec cette instance
	 */
	public String getLevelName() {
		return this.levelname;
	}
	
	/** Retourne une liste des blocs qui representent la carte.
	 * On accde un lment spcifique par getListeBlocs()[y][x]
	 * @return la carte en forme d'un table  deux dimensions
	 */
	public Block[][] getListeBlocs() {
		return this.carte;
	}
}
