package bombmapper;

import java.awt.BorderLayout;
import java.awt.Canvas;
import java.awt.Container;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.Vector;

import javax.help.HelpBroker;
import javax.help.HelpSet;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.KeyStroke;
import javax.swing.border.Border;
import javax.swing.undo.UndoableEdit;

/**
 * @author Sebastian Gfeller, Andreas Jaggi
 * @created Jun 15, 2004
 * @version LevelView.java,v 1.25 2004/06/27 20:30:57 sgfeller Exp
 * @copyright Sebastian Gfeller, Andreas Jaggi 2004
 * @license http://opensource.org/licenses/gpl-license.php GNU Public License
 */
public class LevelView extends JFrame implements ComponentListener,MouseListener,MouseMotionListener,WindowListener {
	// On peut largir les blocs, tout calcul est fait avec ces deux variables
	private int largeurBloc;
	private int hauteurBloc;
	
	/** hauteur du menu et de la barre du fentre. Important pour pouvoir
	 * voir tous les blocs sur le place visible de JFrame...
	 */
	private int hauteurControles;
	
	private Image[] imagesBlocs;
	private LevelReader lvl;
	private Canvas cImages;
	/** Pour ne pas avoir a calculer tout le temps la largeur
	 * de la carte. Later on, si on veut implementer des cartes
	 * de differente grandeur et veut que l'utilisateur soit
	 * capable de changer la grandeur, on doit aussi changer ces
	 * variables
	 */
	private int largeurCarte;
	/** La hauteur de la carte en numros des blocs */
	private int hauteurCarte;
	
	// le layout
	private Container cont;
	
	// Le menu
	/** La barre de menus de BombMapper */
	private JMenuBar jmb;
	// les lments du menu
	private JMenu mCommandes;
	private JMenuItem miNouveau;
	private JMenuItem miOuvrir;
	private JMenuItem miFermer;
	private JMenuItem miEnregistrer;
	private JMenuItem miEnregistrerSur;
	private JMenuItem miQuitter;
	
	private JMenu mEdition;
	private JMenuItem miAnnuler;
	private JMenuItem miRetablir;
	private JMenuItem miChangerImagesTerrain;

	private JMenu mAide;
	private JMenuItem miIndexAide;
	
	/** La palette avec les differents terrains a dessiner */
	private JPanel pToolbar;
	private JButton[] boutonsTerrain;
	private JButton bTerrainSol;
	private JButton bTerrainMou;
	private JButton bTerrainMur;
	private JButton bTerrainIndestructible;
	private JButton bBonusFlamme;
	private JButton bBonusBombe;
	private JButton bBonusTelec;
	private JButton bBonusLanceur;
	private JButton bBonusCrane;
	private JButton bBonusPatin;
	
	
	
	/** Pour pouvoir revisiter le dernier changement */
	private int indexUndo = -1;
	/** Le vecteur qui contient les changements fait  cette carte */
	private Vector edit;

	/** Type de terrain  dessiner */
	private int typeTerrainActuel = 0;
	/* Pour pouvoir dessiner un bloc de Terrain, on doit connaitre le premier bloc qu'on etait sur
	 * en commencant l'action
	 */
	/** Coordonn x commencement de l'action */
	private int xstart = -1;
	/** Coordonn y commencement de l'action */
	private int ystart = -1;
	
	/** Cre une fentre JFrame et affiche le contenu de la carte. Dedans, on a une palette
	 * pour choisir les differents terrains  dessiner
	 * @param lvl La carte  afficher
	 */
	public LevelView(LevelReader lvl) {
		super();
		this.lvl = lvl;
		// Calcule hauteur et largeur de la carte
		hauteurCarte = lvl.getListeBlocs().length;
		largeurCarte = lvl.getListeBlocs()[0].length;
		
		// Listeners et paramtres gnrales  la fentre
		this.addComponentListener(this);
		this.addMouseListener(this);
		this.addMouseMotionListener(this);
		this.addWindowListener(this);
		if(lvl.getLevelName() != null) {
			this.setTitle("Editeur Carte: "+lvl.getLevelName());
		} else {
			this.setTitle("Editeur Carte: Fichier sans Nom");
		}
		// Fonctionnalit pour annuler les changements
		edit = new Vector();
		
		// charger les images
		imagesBlocs = chargeImages(lvl.getNomsFichiers());
		
		// le layout
		cont = this.getContentPane();
		cont.setLayout(new BorderLayout());
		
		// Menu pour le programme
		jmb = new JMenuBar();
		populerMenus(jmb);
		this.setJMenuBar(jmb);
		//int hauteurMenu = 48;
		int hauteurMenu = 32;
		
		// Une toolbar pour choisir les differentes types des Terrains
		pToolbar = new JPanel();
		int hauteurPalette = 48;
		//int hauteurPalette = 32;
		populerPalette(pToolbar);
		cont.add(pToolbar, BorderLayout.NORTH);
		
		hauteurControles = hauteurMenu + hauteurPalette;
		
		// cImages = new Canvas();
		
		// dfinir grandeur initiale des blocs
		//largeurBloc = 48;
		largeurBloc = 32;
		//hauteurBloc = 48;
		hauteurBloc = 32;
		
		// alors on sait la grandeur initiale de la fentre
		this.setSize(largeurBloc*lvl.getListeBlocs()[0].length,hauteurBloc*lvl.getListeBlocs().length+16);
		
		this.setVisible(true);
	}
	
	/** Cre une instance de LevelWriter. Si a ne marche pas,
	 * on ecrit une message dans stdout (plus tard: faire avec un fen&ecirc;tre)
	 */
	private void ecrireCarte() {
		try {
			LevelWriter lw = new LevelWriter(lvl.getLevelName(), lvl.getNomsFichiers(), lvl.getListeBlocs());
		} catch (Exception e) {
			System.out.println("Erreur d'\u00E9criture");
		}
	}
	
	/** Choisit un Fichier pour plusieurs applications. Ceci est fait avec un JFileChooser
	 * qui affiche les fichiers qu'on peut ouvrir ou sauvregarder sur. Pour seulement montrer
	 * les fichiers possible, on utilise le filtre FiltreFichiersCarte.
	 * 
	 * @param action 0: ouvrir 1: enregistrer 2: enregistrer sur
	 * @return le chemin du fichier
	 */
	private String choisirFichier(int action) {
		JFileChooser jfc = new JFileChooser();
		
		// a ajouter: un filtre
		jfc.setFileFilter(new FiltreFichiersCarte());
		jfc.setFileHidingEnabled(true);
		jfc.setFileSelectionMode(JFileChooser.FILES_ONLY);
		switch(action) {
			case 0:
				jfc.showOpenDialog(this);
				break;
			case 1:
				jfc.showSaveDialog(this);
				break;
			case 2:
				jfc.showSaveDialog(this);
				break;
			default:
		}
		
		File f = jfc.getSelectedFile();
		return f.toString();
	}
	
	/** Ajoute les lments au menu. C'est plus lisible si on fait
	 * a dans une methode. Ajoute aussi des raccourcis clavier qui marchent
	 * sur toute plate-forme (setAccelerator)
	 * @param jmb
	 */
	public void populerMenus(JMenuBar jmb) {
		MenuActionListener mal = new MenuActionListener();
		// pour des raccourcis clavier: Touche modificateur (Ctrl sur Windows, Apple sur Mac)
		int mod = Toolkit.getDefaultToolkit().getMenuShortcutKeyMask();
		
		mCommandes = new JMenu("Commandes");
		
		/* On cree des shortcuts qui marchent pour toutes plate-formes */
		miNouveau = new JMenuItem("Nouvelle carte...");
		miNouveau.addActionListener(mal);
		miNouveau.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N,mod));
		mCommandes.add(miNouveau);
		
		miOuvrir = new JMenuItem("Ouvrir un fichier...");
		miOuvrir.addActionListener(mal);
		miOuvrir.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O,mod));
		mCommandes.add(miOuvrir);
		
		miFermer = new JMenuItem("Fermer cette carte");
		miFermer.addActionListener(mal);
		miFermer.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_W,mod));
		mCommandes.add(miFermer);
		
		miEnregistrer = new JMenuItem("Enregistrer");
		miEnregistrer.addActionListener(mal);
		miEnregistrer.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S,mod));
		mCommandes.add(miEnregistrer);
		
		miEnregistrerSur = new JMenuItem("Enregistrer sur...");
		miEnregistrerSur.addActionListener(mal);
		mCommandes.add(miEnregistrerSur);
		
		miQuitter = new JMenuItem("Quitter");
		miQuitter.addActionListener(mal);
		miQuitter.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q,mod));
		mCommandes.add(miQuitter);
		jmb.add(mCommandes);
		
		mEdition = new JMenu("\u00C9dition");
		miAnnuler = new JMenuItem("Annuler");
		miAnnuler.addActionListener(mal);
		miAnnuler.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z,mod));
		mEdition.add(miAnnuler);
		
		miRetablir = new JMenuItem("R\u00E9tablir");
		miRetablir.addActionListener(mal);
		miRetablir.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R,mod));
		mEdition.add(miRetablir);
		
		miChangerImagesTerrain = new JMenuItem("Changer images de terrain...");
		miChangerImagesTerrain.addActionListener(mal);
		miChangerImagesTerrain.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_T,mod));
		mEdition.add(miChangerImagesTerrain);
		jmb.add(mEdition);

		mAide = new JMenu("Aide");
		miIndexAide = new JMenuItem("Guide de l'utilisateur");
		miIndexAide.addActionListener(mal);
		miIndexAide.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_I,mod));
		mAide.add(miIndexAide);
		jmb.add(mAide);
	}
	
	/** Ajoute la palette des types de terrain au JPanel
	 * spcifi.
	 * J'utilise la methode brute force pour ajouter les lments.
	 * Cela me permet de dfinir des Tooltips pour toutes les terrains.
	 * @param pToolbar le JPanel ? populer
	 */
	public void populerPalette(JPanel pToolbar) {
		String[] nfichiers = lvl.getNomsFichiers();
		ImageIcon[] imagesBoutons = new ImageIcon[10];
		
		for(int i=0;i<10;i++) {
				imagesBoutons[i] = new ImageIcon(ResourceLoader.getResource(lvl.getNomsFichiers()[i]));
		}

		ChangeTypeTerrainListener cttlist = new ChangeTypeTerrainListener();
		
		// Pour que les icones ne deviennent pas trop grand, on leur donne un bord vide.
		Border empty = BorderFactory.createEmptyBorder();
		
		bTerrainSol = new JButton(imagesBoutons[0]);
		bTerrainSol.setToolTipText("Terrain libre \u00E9 passer");
		bTerrainSol.addActionListener(cttlist);
		bTerrainSol.setBorder(empty);
		pToolbar.add(bTerrainSol);
		
		bTerrainMou = new JButton(imagesBoutons[1]);
		bTerrainMou.setToolTipText("Terrain qu'on peut faire sauter");
		bTerrainMou.addActionListener(cttlist);
		bTerrainMou.setBorder(empty);
		pToolbar.add(bTerrainMou);
		
		bTerrainMur = new JButton(imagesBoutons[2]);
		bTerrainMur.setToolTipText("Terrain mur");
		bTerrainMur.addActionListener(cttlist);
		bTerrainMur.setBorder(empty);
		pToolbar.add(bTerrainMur);
		
		bTerrainIndestructible = new JButton(imagesBoutons[3]);
		bTerrainIndestructible.setToolTipText("Terrain indestructible");
		bTerrainIndestructible.addActionListener(cttlist);
		bTerrainIndestructible.setBorder(empty);
		pToolbar.add(bTerrainIndestructible);
		
		bBonusFlamme = new JButton(imagesBoutons[4]);
		bBonusFlamme.setToolTipText("Bonus pour une plus grande explosion");
		bBonusFlamme.addActionListener(cttlist);
		bBonusFlamme.setBorder(empty);
		pToolbar.add(bBonusFlamme);
		
		bBonusBombe = new JButton(imagesBoutons[5]);
		bBonusBombe.setToolTipText("Bonus: On a une bombe de plus  placer");
		bBonusBombe.addActionListener(cttlist);
		bBonusBombe.setBorder(empty);
		pToolbar.add(bBonusBombe);
		
		bBonusTelec = new JButton(imagesBoutons[6]);
		bBonusTelec.setToolTipText("Bonus qui fait je ne sais pas...");
		bBonusTelec.addActionListener(cttlist);
		bBonusTelec.setBorder(empty);
		pToolbar.add(bBonusTelec);
		
		bBonusLanceur = new JButton(imagesBoutons[7]);
		bBonusLanceur.setToolTipText("Bonus: On peut lancer la bombe");
		bBonusLanceur.addActionListener(cttlist);
		bBonusLanceur.setBorder(empty);
		pToolbar.add(bBonusLanceur);
		
		bBonusCrane = new JButton(imagesBoutons[8]);
		bBonusCrane.setToolTipText("Malus: on devient plus lent");
		bBonusCrane.addActionListener(cttlist);
		bBonusCrane.setBorder(empty);
		pToolbar.add(bBonusCrane);
		
		bBonusPatin = new JButton(imagesBoutons[9]);
		bBonusPatin.setToolTipText("Bonus qui fait aller plus vite le joueur");
		bBonusPatin.addActionListener(cttlist);
		bBonusPatin.setBorder(empty);
		pToolbar.add(bBonusPatin);
	}
	
	/** Retourne si on a change le fichier, en utilisant la variable indexUndo
	 */
	public boolean hasChanges() {
		return indexUndo>-1;
	}
	
	/** Ferme cette fenetre */
	public void fermerFenetre() {
		this.setVisible(false);
		this.dispose();
	}
	
	/** Ferme la fenetre si on n'a pas chang quelque chose dedans. Si on a,
	 * on presente un dialogue ou on peut choisir si on veut enregistrer les
	 * changements.
	 */
	private void fermerSiSansChangements() {
		if(hasChanges()) {
			int choix = JOptionPane.showConfirmDialog(this,
					"Voulez vous enregistrer les changements fait \u00E0 cette carte?",
					"Enregistrer?",JOptionPane.YES_NO_CANCEL_OPTION);
			if(choix == JOptionPane.YES_OPTION && !(lvl.getLevelName() == null)) {
				ecrireCarte();
				fermerFenetre();
			} else if(choix == JOptionPane.YES_OPTION && lvl.getLevelName() == null) {
				/* On n'a pas encore specifie un nom pour le fichier */
				dialogueEnregistrerSur(); // ouvre encore une fenetre avec le titre comme nom de nouveau fichier.
				fermerFenetre(); // on ferme cette fenetre aussi
			} else if(choix == JOptionPane.NO_OPTION) {
				fermerFenetre();
			}
		}
	}
	
	/** Fait ouvrir un dialogue pour choisir un fichier dont on sauvregarde la carte.
	 * L'ancienne LevelView est cach, on ouvre une nouvelle avec le fichier sauvregarde.
	 */
	protected void dialogueEnregistrerSur() {
		String lnom = choisirFichier(2);
		try {
			LevelWriter carteAEcrire = new LevelWriter(lnom, lvl.getNomsFichiers(), lvl.getListeBlocs());
			fermerFenetre();
			LevelReader nouvelNomCarteOuvert = new LevelReader(lnom);
			LevelView nouvelNomCarteOuvertVue = new LevelView(nouvelNomCarteOuvert);
		} catch (Exception ioe) {
			System.out.println("Erreur entre/sortie");
		}
	}

	/** Affiche un dialogue pour choisir un fichier  ouvrir et essaie de l'ouvrir
	 * dans une nouvelle fentre
	 */
	protected void dialogueOuvrir() {
		String lnom = choisirFichier(0); // 0 = open
		try {
			LevelReader ouvrirCarte = new LevelReader(lnom);
			LevelView ouvrirVueCarte = new LevelView(ouvrirCarte);
		} catch (Exception ioe) {
			System.out.println("Erreur entre/sortie");
		}
	}
	
	/** Cree une nouvelle carte. On ne doit pas specifier un nom,
	 * on cree seulement une instance avec une carte vide et une LevelView pour cette instance.
	 */
	protected void creerNouvelleCarte() {
		/* Un nouveau LevelReader, cre une nouvelle carte */
		try {
			LevelReader nouvelleCarte = new LevelReader();
			LevelView nouvelleVueCarte = new LevelView(nouvelleCarte);
		} catch(Exception ioe) { // Ca n'a pas march. Ne doit pas arriver...
			ioe.printStackTrace();
		}
	}
	
	/** Charge les images spcifes dans une table des Strings
	 * et les retourne en forme d'une autre table
	 * 
	 * @param nfich table des chemins des images
	 * @return table des images
	 */
	private Image[] chargeImages(String[] nfich) {
		Image[] iBlocs = new Image[nfich.length];
		
		for(int i=0;i<nfich.length;i++) {
			iBlocs[i] = new ImageIcon(ResourceLoader.getResource(nfich[i])).getImage();
		}
		
		return iBlocs;
	}

	/** Cette methode rudimentaire affiche les images des blocs
	 * en utilisant les donnes du type dans lvl.getListeBlocs
	 * et les images dans imagesBlocs prchargs avant
	 */
	public void paint(Graphics g) {
		// premirement, reafficher la MenuBar et la palette
		
		jmb.repaint();
		pToolbar.repaint();
		
		Block[][] carte = lvl.getListeBlocs();
		
		try {
			for(int y=0;y<carte.length;y++) {
				for(int x=0;x<carte[y].length;x++) {
					g.drawImage(imagesBlocs[carte[y][x].getType()],
							x*largeurBloc,y*hauteurBloc+hauteurControles,largeurBloc,hauteurBloc,this);
				}
			}
		} catch (NullPointerException e) { 
			/* fichiers images pas encore prsents
			 try again un peu plus tard */
		}
	}
	
	/** Extrait la position horizontale dans la carte representant
	 * ce point sur le JFrame
	 */
	private int extraireX(int xFenetre) {
		return xFenetre/largeurBloc; // roundoff expecte
	}
	
	/** Extrait la position verticale dans la carte representant
	 * ce point sur le JFrame
	 */
	private int extraireY(int yFenetre) {
		return (yFenetre-hauteurControles)/hauteurBloc; // roundoff expecte
	}
	
	/** Change le terrain  une certaine position de la souris.
	 * @param mevt L'evnement de souris qui est la raison pour l'action
	 */
	private void changerTerrainSousSouris(MouseEvent mevt) {
		int x,y;
		x = extraireX(mevt.getX());
		y = extraireY(mevt.getY());
		if (x>0 && y>0 && x<(largeurCarte - 1) && y<(hauteurCarte - 1)) {
			// on change le terrain seulement s'il na pas dja la valeur du typeTerrainActuel
			if(lvl.getListeBlocs()[y][x].getType() != typeTerrainActuel) {
				incrementerIndexUndo();
				edit.add(new ChangeTerrainEdit(lvl.getListeBlocs()[y][x], typeTerrainActuel));	
				// Seulement redessiner le bloc chang: Plus convenient, plus vite
				this.getGraphics().drawImage(imagesBlocs[typeTerrainActuel],
						x*largeurBloc,y*hauteurBloc+hauteurControles,largeurBloc,hauteurBloc,this);
			}
		}
	}
	
	/** On a fait un changement. Les undos apres cet index deviennent inutiles et sont limins.
	 */
	private void incrementerIndexUndo() {
		if(indexUndo < edit.size() - 1) {
			for(int i=edit.size()-1;i>indexUndo;i--) {
				edit.remove(i);
			}
		}
		indexUndo++;
	}

	/** Change un bloc de terrain, finissant avec la position de la souris
	 * @param mevt L'vnement de souris qui est la raison pour l'action
	 */
	private void changeBlocTerrain(MouseEvent mevt) {
		int xstop,ystop;
		int xde,xa,yde,ya;
		xstop = extraireX(mevt.getX());
		ystop = extraireY(mevt.getY());
		
		// plus tard: implementer a dans l'edit
		if(xstart == xstop && ystart == ystop) { // on est sur la meme position
			// on ne fait rien
		} else { // on n'est plus sur la meme position
			// faire l'action actuelle
			incrementerIndexUndo();
			edit.add(new ChangeBlocTerrainEdit(lvl.getListeBlocs(),typeTerrainActuel,xstart,ystart,xstop,ystop));
			repaint();
		}
		// on est fini, pour une nouvelle action on libre xstart et ystart
		xstart = -1;
		ystart = -1;
	}

	
	public void componentHidden(ComponentEvent e) {	repaint(); }
	public void componentMoved(ComponentEvent e) { repaint(); }
	public void componentShown(ComponentEvent e) { repaint(); }
	
	/** Si on change la grandeur de la fentre, cette methode change largeur et hauteur des blocs
	 * pour qu'on peut toujours voir tous les blocs dans la fentre.
	 */
	public void componentResized(ComponentEvent e) {
		largeurBloc = this.getWidth()/largeurCarte;
		hauteurBloc = (this.getHeight()-hauteurControles)/hauteurCarte;
		
		update(this.getGraphics());
	}
	


	/** Si on a cliqu quelque part, on ajoute un ChangeTerrainEdit.
	 * Cet edit peut etre revisite avec le menu undo/redo (plus
	 * tard).
	 */
	public void mouseClicked(MouseEvent mevt) {
		changerTerrainSousSouris(mevt);
	}
	
	/** Si on commence a glisser avec la souris, on enregistre les coordonns x
	 * et y de la souris.
	 * On fait l'action si on a fini a glisser.
	 */
	public void mousePressed(MouseEvent mevt) {
		if(xstart<= 0 || ystart<= 0) { //on est au commencement de l'action: Aussi considr: Le bord de la carte ne compte pas comme commencement
			xstart = extraireX(mevt.getX());
			ystart = extraireY(mevt.getY());
		} // on ne fait rien si ce n'est pas le commencement de l'action
	}
	
	/** On a fini le glissement de la souris. On fait l'action de changement de blocs.
	 * Je trouve plus intelligent de meme dessiner une seule ligne ou une seule colonne
	 * des blocs, mais on peut changer a simplement si on change la condition
	 * if(xstart == xstop && ystart == ystop) 
	 * if(xstart == xstop || ystart == ystop)
	 * et tout >=  > dans else
	 */
	public void mouseReleased(MouseEvent mevt) {
		changeBlocTerrain(mevt);
	}
	

	private class ChangeTypeTerrainListener implements ActionListener {

		public void actionPerformed(ActionEvent e) {
			if(e.getSource() == bTerrainSol) {
				typeTerrainActuel = 0;
			} else if (e.getSource() == bTerrainMou) {
				typeTerrainActuel = 1;
			} else if (e.getSource() == bTerrainMur) {
				typeTerrainActuel = 2;
			} else if (e.getSource() == bTerrainIndestructible) {
				typeTerrainActuel = 3;
			} else if (e.getSource() == bBonusFlamme) {
				typeTerrainActuel = 4;
			} else if (e.getSource() == bBonusBombe) {
				typeTerrainActuel = 5;
			} else if (e.getSource() == bBonusTelec) {
				typeTerrainActuel = 6;
			} else if (e.getSource() == bBonusLanceur) {
				typeTerrainActuel = 7;
			} else if (e.getSource() == bBonusCrane) {
				typeTerrainActuel = 8;
			} else if (e.getSource() == bBonusPatin) {
				typeTerrainActuel = 9;
			}
		}
		
	}
	
	private class MenuActionListener implements ActionListener {
		
		public void actionPerformed(ActionEvent e) {
			if(e.getSource() == miEnregistrer) {
				// si on a un nom pour enregistrer sur, on enregistre l, autrement on ouvre un dialogue
				if(lvl.getLevelName() != null) {
					ecrireCarte();
				} else {
					dialogueEnregistrerSur();
				}
			} else if(e.getSource() == miEnregistrerSur) {
				dialogueEnregistrerSur();
			} else if(e.getSource() == miOuvrir) {
				dialogueOuvrir();
			} else if(e.getSource() == miNouveau) {
				creerNouvelleCarte();
			} else if(e.getSource() == miFermer) {
				fermerSiSansChangements();
			} else if(e.getSource() == miQuitter) {
				// d'abord: voir si on a fait des changements
				fermerSiSansChangements();
				System.exit(0);
			} else if(e.getSource() == miAnnuler) {
				if(indexUndo > 0 && edit.get(indexUndo) != null && ((UndoableEdit)edit.get(indexUndo)).canUndo()) {
					((UndoableEdit)edit.get(indexUndo)).undo();
					indexUndo--;
					repaint();
				}
			} else if(e.getSource() == miRetablir) {
				if((indexUndo < edit.size() - 1) && edit.get(indexUndo) != null && ((UndoableEdit)edit.get(indexUndo)).canRedo()) {
					if(indexUndo < -1) { indexUndo = -1; }
					((UndoableEdit)edit.get(indexUndo+1)).redo();
					indexUndo++;
					repaint();
				}
			} else if(e.getSource() == miChangerImagesTerrain) {
				try {
					/* Ajoute l'action de changement de fichiers au vecteur des changements. Le nouveau
					 * index du changement momentane est le dernier element (autres Undos ne sont plus
					 * applicables)
					 */
					edit.add(new ChangerImagesTerrainEdit(lvl.getNomsFichiers(), imagesBlocs, getThis()));
					indexUndo = edit.size()-1;
					repaint();
				} catch (IOException ioe) { // on a eu un problme avec trouver les fichiers images
					System.out.println("Fichiers images pas trouv\u00E9s.");
				}
			} else if(e.getSource() == miIndexAide) {
				try {
					URL hsURL = new URL("jar:"+ResourceLoader.getResource("help/bombhelp.hs").getFile());
					HelpSet hs = new HelpSet(null, hsURL);
					HelpBroker hb = hs.createHelpBroker();
					hb.setDisplayed(true);
				
				} catch (Exception ee) {
					System.out.println("Erreur d'affichage de l'aide");
				}
			}
		}
	}

	
	private LevelView getThis() {
		return this;
	}
	
	// Methodes vides pour les autres evnements de la souris
	public void mouseDragged(MouseEvent mevt) {	}
	public void mouseEntered(MouseEvent mevt) {	}
	public void mouseExited(MouseEvent mevt) { }
	public void mouseMoved(MouseEvent mevt) { }

	// methodes pour les evenements de la fenetre
	/** Si on ferme la fentre, on demande d'abord si on veut enregistrer les changements
	 * @see java.awt.event.WindowListener#windowClosing(java.awt.event.WindowEvent)
	 */
	public void windowClosing(WindowEvent we) {
		fermerSiSansChangements();
	}

	public void windowOpened(WindowEvent we) { }
	public void windowClosed(WindowEvent we) { }
	public void windowIconified(WindowEvent we) { }
	public void windowDeiconified(WindowEvent we) { repaint(); }
	public void windowActivated(WindowEvent we) { }
	public void windowDeactivated(WindowEvent we) { }
}
